#include "Arduino.h"
#include <inttypes.h>
#include "lpt.h"

//*************************************************************************************
LPT::LPT() {}

//*************************************************************************************
void LPT::c74hc595Out(const byte data) {
  // no speed optimizations. plattformindependant.

  digitalWrite(_ST_CP, LOW);

  for (short i = 7; i >= 0; i--)  {
    digitalWrite(_SH_CP, LOW);

    if ( data & (1 << i) ) {
      digitalWrite(_DS, HIGH);
    }
    else {
      digitalWrite(_DS, LOW);
    }

    digitalWrite(_SH_CP, HIGH);
  }
  digitalWrite(_SH_CP, LOW);

  digitalWrite(_ST_CP, HIGH);
}

//*************************************************************************************
boolean LPT::isbusy(void) {
  return digitalRead(_BUSY) == HIGH;
}

//*************************************************************************************
size_t LPT::write(uint8_t data)
{
  // Warten, dass der Drucker fertig ist.
  unsigned long ticker = millis();
  while ( (millis() - ticker < PtrWaitLimit) && (isbusy()) ) {
    yield();
  }
  if (millis() - ticker >= PtrWaitLimit) {
    // signal "panic now!" to all listening stations
	// Serial.print("WWW");
    return 0; // nix druck
  }

  // shift the data out
  c74hc595Out(data);

  digitalWrite(_STROBE, LOW);
  delay(PrtStrobeTime);
  digitalWrite(_STROBE, HIGH);

  return 1;
}

//*************************************************************************************
void LPT::begin(uint8_t DS, uint8_t ST_CP, uint8_t SH_CP, uint8_t STROBE, uint8_t BUSY)
{
  _DS = DS;
  _ST_CP = ST_CP;
  _SH_CP = SH_CP;
  _STROBE = STROBE;
  _BUSY = BUSY;

  // Set strobe correctly
  pinMode(_STROBE, OUTPUT);
  digitalWrite(_STROBE, HIGH);

  pinMode(_DS, OUTPUT);
  pinMode(_ST_CP, OUTPUT);
  pinMode(_SH_CP, OUTPUT);
  pinMode(_BUSY, INPUT);
  
  c74hc595Out(0);
}